/***************************************************************************
 *   Copyright (C) 2009 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "cdemu.h"

#include <QFile>
#include <QtDBus>

#include <iostream>
using namespace std;

//---------------------------------------------------------------------------------------------------------- //
//    Construction & destruction                                                                             //
//---------------------------------------------------------------------------------------------------------- //

CDEmu::CDEmu()
    : m_sessionBus(QDBusConnection::sessionBus()),
      m_systemBus(QDBusConnection::systemBus()),
      m_currentBus(0)
{
    this->findCurrentBus();

    this->connectSlots(&m_sessionBus);
    this->connectSlots(&m_systemBus);
}

//---------------------------------------------------------------------------------------------------------- //

CDEmu::~CDEmu()
{
}


//---------------------------------------------------------------------------------------------------------- //
//    Information access                                                                                     //
//---------------------------------------------------------------------------------------------------------- //

bool CDEmu::daemonRunning()
{
    return (m_currentBus != 0);
}

//---------------------------------------------------------------------------------------------------------- //

int CDEmu::numDevices()
{
    if (!this->daemonRunning())
        return -1;

    QDBusInterface interface("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                             "net.sf.cdemu.CDEMUD_Daemon", *m_currentBus, this);

    QDBusReply<int> reply = interface.call("GetNumberOfDevices");

    if (reply.isValid())
        return reply.value();
    else
    {
        cerr << "Unable to detect number of virtual drives!" << endl;
        return -1;
    }
}

//---------------------------------------------------------------------------------------------------------- //

bool CDEmu::loaded(unsigned int device)
{
    if (!this->daemonRunning())
        return false;

    QDBusInterface interface("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                             "net.sf.cdemu.CDEMUD_Daemon", *m_currentBus, this);

    QDBusMessage reply = interface.call("DeviceGetStatus", (int)device);

    if (reply.type() != QDBusMessage::ErrorMessage)
    {
        QList<QVariant> args = reply.arguments();
        return args[0].toBool();
    }
    else
    {
        cerr << "Unable to detect status of virtual drive " << device << "!" << endl;
        return false;
    }
}

//---------------------------------------------------------------------------------------------------------- //

QString CDEmu::filename(unsigned int device)
{
    if (!this->daemonRunning())
        return QString("Error!");

    QDBusInterface interface("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                             "net.sf.cdemu.CDEMUD_Daemon", *m_currentBus, this);

    QDBusMessage reply = interface.call("DeviceGetStatus", (int)device);

    if (reply.type() != QDBusMessage::ErrorMessage)
    {
        QList<QVariant> args = reply.arguments();
        return args[1].toList()[0].toString();
    }
    else
        return QString("Error!");
}


//---------------------------------------------------------------------------------------------------------- //
//    Basic operations                                                                                       //
//---------------------------------------------------------------------------------------------------------- //

CDEmu::Result CDEmu::mount(QString filename)
{
    if (!this->daemonRunning())
        return DaemonNotRunning;

    if (!QFile::exists(filename))
        return FileNotFound;

    int slot = this->nextFreeSlot();
    if (slot < 0)
        return NoFreeDevice;

    QStringList filenames;
    filenames << filename;

    QVariantMap parameters; // Unused for now

    QDBusInterface interface("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                             "net.sf.cdemu.CDEMUD_Daemon", *m_currentBus, this);

    QDBusMessage reply = interface.call("DeviceLoad", slot, filenames, parameters);
    if (reply.type() != QDBusMessage::ErrorMessage)
        return Success;

    return UnknownMountError;
}

//---------------------------------------------------------------------------------------------------------- //

CDEmu::Result CDEmu::unmount(unsigned int device)
{
    if (!this->daemonRunning())
        return DaemonNotRunning;

    QDBusInterface interface("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                             "net.sf.cdemu.CDEMUD_Daemon", *m_currentBus, this);

    QDBusMessage reply = interface.call("DeviceUnload", (int)device);
    if (reply.type() != QDBusMessage::ErrorMessage)
        return Success;

    return UnknownUnmountError;
}


//---------------------------------------------------------------------------------------------------------- //
//    Private slots                                                                                          //
//---------------------------------------------------------------------------------------------------------- //

void CDEmu::slotDeviceChanged(int i1)
{
    emit deviceChanged(i1);
}

//---------------------------------------------------------------------------------------------------------- //

void CDEmu::slotDaemonChanged()
{
    this->findCurrentBus();
    emit daemonChanged(m_currentBus != 0);
}


//---------------------------------------------------------------------------------------------------------- //
//    Private helpers                                                                                        //
//---------------------------------------------------------------------------------------------------------- //

void CDEmu::connectSlots(QDBusConnection *connection)
{
    connection->connect("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                        "net.sf.cdemu.CDEMUD_Daemon", "DaemonStarted",
                        this, SLOT(slotDaemonChanged()));
                        
    connection->connect("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                        "net.sf.cdemu.CDEMUD_Daemon", "DaemonStopped",
                        this, SLOT(slotDaemonChanged()));
}

//---------------------------------------------------------------------------------------------------------- //

void CDEmu::findCurrentBus()
{
    // We don't want to get signals from the wrong bus in case we
    // switch from the session bus to the system bus or vice versa
    if (m_currentBus)
    {
        m_currentBus->disconnect("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                                 "net.sf.cdemu.CDEMUD_Daemon", "DeviceStatusChanged",
                                 this, SLOT(slotDeviceChanged(int)));
    }

    // Try session bus first
    QDBusConnectionInterface *interface = m_sessionBus.interface();
    if (interface->isServiceRegistered("net.sf.cdemu.CDEMUD_Daemon"))
        m_currentBus = &m_sessionBus;
    else
    {
        // Try system bus then
        interface = m_systemBus.interface();
        if (interface->isServiceRegistered("net.sf.cdemu.CDEMUD_Daemon"))
            m_currentBus = &m_systemBus;
        else
            m_currentBus = 0;
    }

    // Reconnect
    if (m_currentBus)
    {
        m_currentBus->connect("net.sf.cdemu.CDEMUD_Daemon", "/CDEMUD_Daemon",
                              "net.sf.cdemu.CDEMUD_Daemon", "DeviceStatusChanged",
                              this, SLOT(slotDeviceChanged(int)));
    }
}

//---------------------------------------------------------------------------------------------------------- //

int CDEmu::nextFreeSlot()
{
    int numDevices = this->numDevices();
    for (int i = 0; i < numDevices; i++)
    {
        if (!CDEmu::instance()->loaded(i))
            return i;
    }

    return -1;
}


//---------------------------------------------------------------------------------------------------------- //
//    Public singleton access                                                                                //
//---------------------------------------------------------------------------------------------------------- //

CDEmu* CDEmu::instance()
{
    static CDEmu emu;
    return &emu;
}

//---------------------------------------------------------------------------------------------------------- //
