/***************************************************************************
 *   Copyright (C) 2009 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "cdemu.h"
#include "mainwindow.h"
#include "messagebox.h"

#include <QHeaderView>
#include <QLabel>
#include <QtDBus>

#include <KFileDialog>
#include <KMenu>
#include <KMenuBar>
#include <KMessageBox>
#include <KStandardAction>
#include <KStatusBar>

#include <iostream>
using namespace std;

static QString fileTypes("*.iso *.cue *.toc *.nrg *.mds *.daa *.cif *.b6t *.c2d *.cdi *.ccd");

//---------------------------------------------------------------------------------------------------------- //
//    Construction & destruction                                                                             //
//---------------------------------------------------------------------------------------------------------- //

MainWindow::MainWindow(QWidget *parent) : KXmlGuiWindow(parent)
{
    m_ui.setupUi(this);

    QObject::connect(CDEmu::instance(), SIGNAL(deviceChanged(int)), this, SLOT(slotDeviceChanged(int)));
    QObject::connect(CDEmu::instance(), SIGNAL(daemonChanged(bool)), this, SLOT(slotDaemonChanged(bool)));

    QObject::connect(m_ui.SelectFileButton, SIGNAL(clicked()), this, SLOT(slotSelectFile()));
    QObject::connect(m_ui.MountButton, SIGNAL(clicked()), this, SLOT(slotMount()));
    QObject::connect(m_ui.UnmountButton, SIGNAL(clicked()), this, SLOT(slotUnmount()));

    KMenu *fileMenu = new KMenu(i18n("File"), this);
    this->menuBar()->addMenu(fileMenu);

    KStandardAction::quit(this, SLOT(close()), (QObject*)actionCollection());

    m_statusLabel = new QLabel(this);
    this->statusBar()->insertWidget(0, m_statusLabel);
    this->slotDaemonChanged(CDEmu::instance()->daemonRunning());

    this->setupGUI(Save | Create);

    m_ui.DeviceList->header()->setStretchLastSection(false);

    m_ui.DeviceList->header()->setResizeMode(0, QHeaderView::Fixed);
    m_ui.DeviceList->header()->setResizeMode(1, QHeaderView::Stretch);

    m_ui.DeviceList->header()->resizeSection(0, QFontMetrics(this->font()).width(i18n("Drive") + "xxx"));

    this->show();
}

//---------------------------------------------------------------------------------------------------------- //

MainWindow::~MainWindow()
{
}


//---------------------------------------------------------------------------------------------------------- //
//    Private slots                                                                                          //
//---------------------------------------------------------------------------------------------------------- //

void MainWindow::slotDeviceChanged(int device)
{
    QString filename;
    if (CDEmu::instance()->loaded(device))
        filename = CDEmu::instance()->filename(device);

    m_ui.DeviceList->topLevelItem(device)->setText(1, filename);
}

//---------------------------------------------------------------------------------------------------------- //

void MainWindow::slotDaemonChanged(bool running)
{
    m_ui.DeviceList->setEnabled(running);
    m_ui.SelectImageBox->setEnabled(running);
    m_ui.MountButton->setEnabled(running);
    m_ui.UnmountButton->setEnabled(running);

    m_ui.DeviceList->clear();

    if (running)
    {
        int numDevices = CDEmu::instance()->numDevices();
        if (numDevices >= 0)
        {
            for (int i = 0; i < numDevices; i++)
            {
                QStringList strings;
                strings << QString().setNum(i);
                if (CDEmu::instance()->loaded(i))
                    strings << CDEmu::instance()->filename(i);
                QTreeWidgetItem *item = new QTreeWidgetItem(strings);
                m_ui.DeviceList->addTopLevelItem(item);
            }
        }

        m_statusLabel->setText(i18n("CDEmu daemon running."));
    }
    else
        m_statusLabel->setText(i18n("CDEmu daemon not running."));
}

//---------------------------------------------------------------------------------------------------------- //

void MainWindow::slotSelectFile()
{
    QString file = KFileDialog::getOpenFileName(KUrl(), fileTypes, this, QString());
    m_ui.FileNameEdit->setText(file);
}

//---------------------------------------------------------------------------------------------------------- //

void MainWindow::slotMount()
{
    if (m_ui.FileNameEdit->text().isEmpty())
    {
        KMessageBox::information(this, i18n("Please select a file first."), i18n("Error"));
        return;
    }

    CDEmu::Result result = CDEmu::instance()->mount(m_ui.FileNameEdit->text());
    if (result != CDEmu::Success)
        MessageBox::error(result);
    m_ui.FileNameEdit->clear();
}

//---------------------------------------------------------------------------------------------------------- //

void MainWindow::slotUnmount()
{
    int device = m_ui.DeviceList->indexOfTopLevelItem(m_ui.DeviceList->currentItem());
    if (device < 0)
    {
        KMessageBox::information(this, i18n("Please select a device first."), i18n("Error"));
        return;
    }

    CDEmu::Result result = CDEmu::instance()->unmount((unsigned int)device);
    if (result != CDEmu::Success)
        MessageBox::error(result);
}

//---------------------------------------------------------------------------------------------------------- //
